module FlightSims

# using Debugger  # tmp
using DifferentialEquations
using ComponentArrays, UnPack
using JLD2, FileIO, DataFrames
using SplitApplyCombine  # e.g., SplitApplyCombine.group
using LinearAlgebra, Transducers, Rotations, Random, ForwardDiff
using Combinatorics: multiexponents
using NLopt
using NumericalIntegration: integrate
using DynamicPolynomials: @polyvar, PolyVar, AbstractPolynomialLike
using Flux
using Flux.Data: DataLoader
using MatrixEquations

### APIs
export AbstractEnv, State, Params, Dynamics, Dynamics!, apply_inputs, DatumFormat, save_inputs
export sim, Process, load
### envs
## basics
export TwoDimensionalNonlinearPolynomialEnv, LinearSystemEnv, ReferenceModelEnv
## controllers
export LQR
export BacksteppingPositionControllerEnv
## multicopters
export IslamQuadcopterEnv, GoodarziQuadcopterEnv
export LeeHexacopterEnv
# faults
export FaultSet, LoE
# mixers
export PseudoInverseMixer
### algorithms
export command
export CTValueIterationADP, BehaviouralCloning
### utils
export AbstractApproximator, LinearApproximator
export PolynomialBasis, euler
export PowerLoop, command_generator


include("utils/utils.jl")
include("environments/environments.jl")
include("APIs/APIs.jl")
include("algorithms/algorithms.jl")


end
